/*
* Copyright (C) 2002 Alain Stucki <astucki@myisp.ch>
*                    Roman Senn   <roman@dreamlab.ch>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Id: configfile.c,v 1.10 2002/07/31 21:17:55 astucki Exp $
*/

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <netdb.h>
#include <netinet/in.h>
#include <setjmp.h>
#include <sys/socket.h>
#include <sys/stat.h>	/* for umask() on some systems */
#include <sys/types.h>
#include <sys/time.h>

#include "services.h"

/* Setting up the config struct */
config_t config[] = {   /* Defines all posible configs */
    {"MYSQLHOST"},
    {"MYSQLUSER"},
    {"MYSQLPASS"},
    {"MYSQLDB"},
    {"MYSQLPORT"},
    {"LOGFILE"},
    {"PIDFILE"},
    {"CONFIGFILE"},
    {"SSL"}
  
  /* The "Mainconfig" will be readed later from MySQL Server */
};





/*========================*/
/* Read config from MySQL */
/*========================*/
int readmysqlconf()
{
  MYSQL_RES *result;
  MYSQL_ROW sqlrow;
  int i = 0;
  
  if(!query("select name, value from config limit 0,%d", sizeof(MySQLconfig)))
    {
      return 0;
    }
  else
    {
      result = database_result();
      if (mysql_num_rows(result) > sizeof(MySQLconfig))
        fatal("to0 much configvalues selected. don't try to overflow my config struct");
      
      while((sqlrow = mysql_fetch_row(result)))
        {
          debugmsg(1, "Adding MySQL Config to Struct: Name: %s Value: %s", sqlrow[0], sqlrow[1]);
          pushmysqlconf(sqlrow[0], sqlrow[1], i);
          i++;
        }
      return 1;
    }
  
  
}

/*=====================================*/
/* Push a config into mysqlconf struct */
/*=====================================*/
void pushmysqlconf(char *key, char *value, int index)
{
  strncpy(MySQLconfig[index].key, key, 255);
  strncpy(MySQLconfig[index].value, value, 255);
  
}

/*==========================================================*/
/* Dump all entrys from mysqlconf struct (for debuging only */
/*==========================================================*/
void dumpmysqlconf()
{
  unsigned int i;
  char *key;
  char *value;
  for (i = 0; i < MAXCONF; i++)
    {
      key = MySQLconfig[i].key;
      value = MySQLconfig[i].value;
      
      if (strlen(key) > 0 && strlen(value) > 0)
        debugmsg(100, "MySQLconfig entry #%d: Key -> %s, Value -> %s", i, key, value);
      
    }
}

char *getmysqlconfigvalue(char *key)
{
  unsigned int i;
  char *temp_key;
  char *temp_value;
  
  
  for (i = 0; i < MAXCONF; i++)
    {
      temp_key = MySQLconfig[i].key;
      temp_value = MySQLconfig[i].value;
      if (strcmp(key, temp_key) == 0)
        return temp_value;
      
    }
  return NULL;
}
/*=======================*/
/* Read config from File */
/*=======================*/
int readconfig()
{
  
  char buffer[4096];
  FILE *fd;
  unsigned int i;
  
  fd = fopen(getconfigvalue("CONFIGFILE"), "r");
  if (!fd)
    {
      seterror("%s", strerror(errno));
      return 0;
    }
  while(!feof(fd))
    {
      fgets(buffer, 4096, fd);
      for (i = 0; i < sizeof(config) / sizeof(config_t); i++)
        {
          char *value;
          value = strchr(buffer, ':');
          if (!value)
            continue;
          *value++ = '\0';
          
          if (strlen(getconfigvalue(buffer)) == 0)
            {
              if (strcmp(config[i].key, buffer) == 0)
                {
                  if (value[strlen(value) - 1] == '\n')
                    value[strlen(value) - 1] = '\0';
                  
                  strncpy(config[i].value, value, 255);
                } else {
                  *(value - 1) = ':';
                }
            }
        }
      
    }
  fclose(fd);
  return 1;
}


/*====================================================*/
/* Reads a Configvalue (from config_t struct) by Name */
/*====================================================*/
char *getconfigvalue(char *key)
{
  int i;
  
  for (i = 0; i < sizeof(config) / sizeof(config_t); i++)
    {
      if (strcmp(config[i].key, key) == 0)
        {
          return config[i].value;
        }
    }
  return NULL;
}

/*=================================================*/
/* Push a Configvalue (by key) to config_t struct) */
/*=================================================*/
int push_configvalue(char *key, char *value)
{
  int i;
  
  for (i = 0; i < sizeof(config) / sizeof(config_t); i++)
    {
      if (strcmp(config[i].key, key) == 0)
        {
          if (value[strlen(value) - 1] == '\n')
            value[strlen(value) - 1] = '\0';
          
          strncpy(config[i].value, value, 255);
          return 1;
        }
    }
  fprintf(stderr, "can't write to config struct");
  return 0;
}
