/*
* Copyright (C) 2002 Alain Stucki <astucki@myisp.ch>
*                    Roman Senn   <roman@dreamlab.ch>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Id: import_export.h,v 1.8 2002/08/03 20:04:17 astucki Exp $
*/

#include <openssl/ssl.h>


/*=================
 * SSL 
 */

extern SSL *ssl_init();
SSL *ssl_init();

/*==================*/
/* Export functions */
/*==================*/
extern int parse_options(int ac, char **av);
extern void sighup_handler(int sig_unused);
extern MYSQL sql;
extern int database_connect();
extern int query(const char *format, ...);
extern MYSQL_RES *database_result();
extern int readconfig();
extern int push_configvalue(char *key, char *value);
extern char *getconfigvalue(char *key);
extern void warning(const char *fmt, ...);
extern void error(const char *fmt, ...);
extern void fatal(const char *fmt, ...);
extern void notice(const char *fmt, ...);
extern void printok();
extern void printfail();
extern void printwarn();
extern void message(const char *fmt, ...);
extern void seterror(const char *fmt, ...);
extern void debugmsg(int level, const char *fmt, ...);
extern void log(const char *fmt, ...);
extern int open_log(void);
extern int writepid(int pid);
extern int readmysqlconf();
extern void pushmysqlconf(char *key, char *value, int index);
extern void dumpmysqlconf();
extern int conn(const char *host, int port);
extern char *getmysqlconfigvalue(char *key);
extern void send_cmd(const char *source, const char *fmt, ...);
extern void vsend_cmd(const char *source, const char *fmt, va_list args);
extern char *sgets(char *buf, int len, int s);
extern int stricmp(const char *s1, const char *s2);
extern int strnicmp(const char *s1, const char *s2, size_t len);
extern char *sgets2(char *buf, int len, int s);
extern void process();
extern int do_nick(const char *source, int ac, char **av);
extern char *strscpy(char *d, const char *s, size_t len);
extern void nickserv_message(char *from, char *message);
extern int split_buf(char *buf, char ***argv_ptr, int colon_special);
extern void nickserv_changenick(char *nick, char *newnick, time_t timestamp);
extern struct NickServ_Timer *gettimerptr(char *name);
extern struct NickServ_Timer *fetch_nickservtimer();
extern void destroy_timer(struct NickServ_Timer *timer);
extern void addkilltimer(char *nick);
extern void nickserv_check_kill_timer();
extern void nickserv_newnick(char *nick);
/*==================*/


/*==================*/
/* Import functions */
/*==================*/
int do_nick(const char *source, int ac, char **av);
void sighup_handler(int sig_unused);
int parse_options(int ac, char **av);
int database_connect();
int database_query(char *query);
MYSQL_RES *database_result();
char *getconfigvalue (char *key);
int readconfig();
int push_configvalue(char *key, char *value);
void warning(const char *fmt, ...);
void error(const char *fmt, ...);
void fatal(const char *fmt, ...);
void notice(const char *fmt, ...);
void printok();
void printfail();
void printwarn();
void message(const char *fmt, ...);
void seterror(const char *fmt, ...);
void debugmsg(int level, const char *fmt, ...);
void log(const char *fmt, ...);
int open_log(void);
int writepid(int pid);
int readmysqlconf();
void pushmysqlconf(char *key, char *value, int index);
void dumpmysqlconf();
int conn(const char *host, int port);
char *getmysqlconfigvalue(char *key);
void send_cmd(const char *source, const char *fmt, ...);
void vsend_cmd(const char *source, const char *fmt, va_list args);
char *sgets(char *buf, int len, int s);
int stricmp(const char *s1, const char *s2);
int strnicmp(const char *s1, const char *s2, size_t len);
char *sgets2(char *buf, int len, int s);
void process();
char *strscpy(char *d, const char *s, size_t len);
void nickserv_message(char *from, char *message);
int split_buf(char *buf, char ***argv_ptr, int colon_special);
void nickserv_changenick(char *nick, char *newnick, time_t timestamp);
struct NickServ_Timer *gettimerptr(char *name);
struct NickServ_Timer *fetch_nickservtimer();
void destroy_timer(struct NickServ_Timer *timer);
void addkilltimer(char *nick);
void nickserv_check_kill_timer();
void nickserv_newnick(char *nick);
/*==================*/


/*==================*/
/* Export variables */
/*==================*/
extern  char    *quitmsg;
extern  int     debug;
extern  char    serr[2048];
extern 	struct 	mysqlconf MySQLconfig[MAXCONF];
extern  int 	sock;
extern  char    *ServerName;
extern 	int     chanserv;
extern 	int     nickserv;
extern	int     memoserv;
/*==================*/

/*==================*/
/* Import variables */
/*==================*/
int     debug;
char    *quitmsg;
char    serr[2048];
struct 	mysqlconf MySQLconfig[MAXCONF];
int 	sock;
char    inbuf[1024];
char    *ServerName;
int     chanserv;
int     nickserv;
int     memoserv;
/*==================*/


