/*
* Copyright (C) 2002 Alain Stucki <astucki@myisp.ch>
*                    Roman Senn   <roman@dreamlab.ch>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Id: init.c,v 1.12 2002/07/31 21:17:55 astucki Exp $
*/
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <netdb.h>
#include <netinet/in.h>
#include <setjmp.h>
#include <sys/socket.h>
#include <sys/stat.h>	/* for umask() on some systems */
#include <sys/types.h>
#include <sys/time.h>

#include "services.h"


/*===========================================*/
/* Parse Commandline options 		     */
/* Commandline options overwrites configfile */
/*===========================================*/
int parse_options(int ac, char **av)
{
  int i;
  char *s;
  
  for (i = 1; i < ac; i++)
    {
      s = av[i];
      if (*s == '-')
        {
          s++;
          if (strcmp(s, "mysqlhost") == 0)
            {
              if (++i >= ac)
                {
                  fprintf(stderr, "-mysqlhost requires hostname[:port]\n");
                  return -1;
                }
              s = av[i];
              if (!push_configvalue("MYSQLHOST", s))
                return -1;
            }
          else if (strcmp(s, "mysqlport") == 0)
            {
              if (++i >= ac)
                {
                  fprintf(stderr, "-mysqlport requires a Port\n");
                  return -1;
                }
              s = av[i];
              if (!push_configvalue("MYSQLPORT", s))
                return -1;
            }
          else if (strcmp(s, "mysqldb") == 0)
            {
              if (++i >= ac)
                {
                  fprintf(stderr, "-mysqldb requires a DB Name\n");
                  return -1;
                }
              s = av[i];
              if (!push_configvalue("MYSQLDB", s))
                return -1;
            }
          else if (strcmp(s, "mysqluser") == 0)
            {
              if (++i >= ac)
                {
                  fprintf(stderr, "-mysqluser requires a Username\n");
                  return -1;
                }
              s = av[i];
              if (!push_configvalue("MYSQLUSER", s))
                return -1;
            } 
          else if (strcmp(s, "mysqlpass") == 0) 
            {
              if (++i >= ac) 
                {
                  fprintf(stderr, "-mysqlpass requires a Password\n");
                  return -1;
                }
              s = av[i];
              if (!push_configvalue("MYSQLPASS", s))
                return -1;
            }
          else if (strcmp(s, "logfile") == 0)
            {
              if (++i >= ac)
                {
                  fprintf(stderr, "-logfile requires a Path to File\n");
                  return -1;
                }
              s = av[i];
              if (!push_configvalue("LOGFILE", s))
                return -1;
            }
          else if (strcmp(s, "pidfile") == 0)
            {
              if (++i >= ac)
                {
                  fprintf(stderr, "-pidfile requires a Path to File\n");
                  return -1;
                }
              s = av[i];
              if (!push_configvalue("PIDFILE", s))
                return -1;
            }
          else if (strcmp(s, "configfile") == 0)
            {
              if (++i >= ac)
                {
                  fprintf(stderr, "-configfile requires a Path to File\n");
                  return -1;
                }
              s = av[i];
              if (!push_configvalue("CONFIGFILE", s))
                return -1;
            }
          else if (strcmp(s, "h") == 0 || strcmp(s, "help") == 0 || strcmp(s, "-help") == 0) 
            {
              fputs("\
The following options are recognized:
                    -mysqlhost 	server		Connect to the specified MySQL Server
                      -mysqlport      port            Set the Port for MySQL Server
                      -mysqldb        db              Select the DB witch should be used by services
                      -mysqluser      username        Set the Username for MySQL Server
                      -mysqlpass      password        Set the Password for MySQL Server
                      -logfile        logfile         Set Path to Logfile
                      -pidfile        pidfile         Set Path to PID-File
                      -configfile     configfile      Set Path to Config File
                      ", stdout);
              exit(0);
            } 
          else 
            {
              fprintf(stderr, "Unknown option -%s.  Use \"-help\" for help.\n", s);
              return -1;
            }
        } 
      else 
        {
          fprintf(stderr, "Non-option arguments not allowed\n");
          return -1;
        }
    }
  return 0;
}

int stricmp(const char *s1, const char *s2)
{
  register int c;
  
  while ((c = tolower(*s1)) == tolower(*s2)) {
    if (c == 0)
      return 0;
    s1++;
    s2++;
  }
  if (c < tolower(*s2))
    return -1;
  return 1;
}

int strnicmp(const char *s1, const char *s2, size_t len)
{
  register int c;
  
  if (!len)
    return 0;
  while ((c = tolower(*s1)) == tolower(*s2) && len > 0) {
	if (c == 0 || --len == 0)
      return 0;
    s1++;
    s2++;
  }
  if (c < tolower(*s2))
    return -1;
  return 1;
}

char *strscpy(char *d, const char *s, size_t len)
{
  char *d_orig = d;
  
  if (!len)
    return d;
  while (--len && (*d++ = *s++))
    ;
  *d = 0;
  return d_orig;
}
