/*
* Copyright (C) 2002 Alain Stucki <astucki@myisp.ch>
*                    Roman Senn   <roman@dreamlab.ch>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Id: log.c,v 1.9 2002/07/31 21:17:55 astucki Exp $
*/
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <netdb.h>
#include <netinet/in.h>
#include <setjmp.h>
#include <sys/socket.h>
#include <sys/stat.h>	/* for umask() on some systems */
#include <sys/types.h>
#include <sys/time.h>

#include "services.h"

static FILE *logfile;

/*===========================*/
/* Open Logfile for Writeing */
/*===========================*/
int open_log(void)
{
  logfile = fopen(getconfigvalue("LOGFILE"), "a");
  if (!logfile)
    {
      seterror("%s", strerror(errno));
      return 0;
    } else {
      setbuf(logfile, NULL);
      return 1;
    }
}

/*===================*/
/* Writeing PID file */
/*===================*/
int writepid(int pid)
{
  FILE *fd;
  char temp[512];
  fd = fopen(getconfigvalue("PIDFILE"), "w");
  if (!fd)
    {
      seterror("%s", strerror(errno));
      return 0;
    } else {
      sprintf(temp, "%d\n", pid);
      fputs(temp, fd);
      fclose(fd);
      return 1;
    }
  
}

/*===============*/
/* Close Logfile */
/*===============*/
void close_log(void)
{
  if (!logfile)
    return;
    fclose(logfile);
  logfile = NULL;
}

/*=================*/
/* Print a Blue ok */
/*=================*/
void printok()
{
  fprintf(stderr, "(\033[1;34mok\033[0;37m)\n");
}

/*====================*/
/* Print a Red failed */
/*====================*/
void printfail()
{
  fprintf(stderr, "(\033[1;31mfailed\033[0;37m)\n");
}

/*=====================*/
/* Print a yellow attn */
/*=====================*/
void printwarn()
{
  fprintf(stderr, "(\033[1;33mattn\033[0;37m)\n");
}

/*=======================*/
/* Print a debug message */
/*=======================*/
void debugmsg(int level, const char *fmt, ...)
{
  va_list args;
  time_t t;
  struct tm tm;
  char buf[2048];
  va_start(args, fmt);
  time(&t);
  tm = *localtime(&t);
  strftime(buf, sizeof(buf)-1, "[%b %d %H:%M:%S %Y] ", &tm);
  if (debug >= level)
    {
      if (logfile)
        {
          fputs(buf, logfile);
                fputs("DEBUG: ", logfile);
          vfprintf(logfile, fmt, args);
          fputc('\n', logfile);
        } else {
		fputs(buf, stderr);
          fputs("DEBUG: ", stderr);
          vfprintf(stderr, fmt, args);
          fputc('\n', stderr);
        }
    }
  va_end(args);
}

/*=================*/
/* Print a message */
/*=================*/
void message(const char *fmt, ...)
{
  va_list args;
  time_t t;
  struct tm tm;
  va_start(args, fmt);
  time(&t);
  tm = *localtime(&t);
  
  vfprintf(stderr, fmt, args);
  va_end(args);
}

/*=====================*/
/* Set a Error Message */
/*=====================*/
void seterror(const char *fmt, ...)
{
  va_list args;
  va_start(args, fmt);
  vsnprintf(serr, 2048, fmt, args);
  va_end(args);
  
}

/*=====================*/
/* Print a Log Message */
/*=====================*/
void log(const char *fmt, ...)
{
  va_list args;
  time_t t;
  struct tm tm;
  char buf[2048];
  va_start(args, fmt);
  time(&t);
  tm = *localtime(&t);
  strftime(buf, sizeof(buf)-1, "[%b %d %H:%M:%S %Y] ", &tm);

  if (logfile)
    {
      fputs(buf, logfile);
      vfprintf(logfile, fmt, args);
      fputc('\n', logfile);
    } else {
      fputs(buf, stderr);
      vfprintf(stderr, fmt, args);
      fputc('\n', stderr);
    }
  va_end(args);
}


/*=================*/
/* Print a Warning */
/*=================*/
void warning(const char *fmt, ...)
{
  va_list args;
  time_t t;
  struct tm tm;
  char buf[2048];
  va_start(args, fmt);
  time(&t);
  tm = *localtime(&t);
  strftime(buf, sizeof(buf)-1, "[%b %d %H:%M:%S %Y] ", &tm);
  
  if (logfile)
    {
      fputs(buf, logfile);
      fputs("WARNING: ", logfile);
      vfprintf(logfile, fmt, args);
      fputc('\n', logfile);
    } else {
      fputs(buf, stderr);
      fputs("WARNING: ", stderr);
      vfprintf(stderr, fmt, args);
      fputc('\n', stderr);
    }
  va_end(args);
}

/*===============*/
/* Print a Error */
/*===============*/
void error(const char *fmt, ...)
{
  va_list args;
  time_t t;
  struct tm tm;
  char buf[2048];
  va_start(args, fmt);
  time(&t);
  tm = *localtime(&t);
  strftime(buf, sizeof(buf)-1, "[%b %d %H:%M:%S %Y] ", &tm);
  
  if (logfile)
    {
      fputs(buf, logfile);
      fputs("ERROR: ", logfile);
      vfprintf(logfile, fmt, args);
      fputc('\n', logfile);
    } else {
      fputs(buf, stderr);
      fputs("ERROR: ", stderr);
      vfprintf(stderr, fmt, args);
      fputc('\n', stderr);
    }
  va_end(args);
}


/*=====================*/
/* Print a Fatal Error */
/*=====================*/
void fatal(const char *fmt, ...)
{
  va_list args;
  time_t t;
  struct tm tm;
  char buf[2048];
  va_start(args, fmt);
  time(&t);
  tm = *localtime(&t);
  strftime(buf, sizeof(buf)-1, "[%b %d %H:%M:%S %Y] ", &tm);
  
  if (logfile) 
    {
      
      fputs(buf, logfile);
      fputs("FATAL ERROR: ", logfile);
      vfprintf(logfile, fmt, args);
      fputc('\n', logfile);
    }
  
  
  fputs(buf, stderr);
  fputs("FATAL ERROR: ", stderr);
  vfprintf(stderr, fmt, args);
  fputc('\n', stderr);
  
  va_end(args);
  exit(666);
}

/*================*/
/* Print a Notice */
/*================*/
void notice(const char *fmt, ...)
{
  va_list args;
  time_t t;
  struct tm tm;
  char buf[2048];
  va_start(args, fmt);
  time(&t);
  tm = *localtime(&t);
  strftime(buf, sizeof(buf)-1, "[%b %d %H:%M:%S %Y] ", &tm);
  
  if (logfile)
    {
      fputs(buf, logfile);
      fputs("NOTICE: ", logfile);
      vfprintf(logfile, fmt, args);
      fputc('\n', logfile);
    } else {
      fputs(buf, stderr);
      fputs("NOTICE: ", stderr);
      vfprintf(stderr, fmt, args);
      fputc('\n', stderr);
    }
  va_end(args);
}
