/*
* Copyright (C) 2002 Alain Stucki <astucki@myisp.ch>
*                    Roman Senn   <roman@dreamlab.ch>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Id: main.c,v 1.27 2002/08/03 20:04:17 astucki Exp $
*/
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <netdb.h>
#include <netinet/in.h>
#include <setjmp.h>
#include <sys/socket.h>
#include <sys/stat.h>	/* for umask() on some systems */
#include <sys/types.h>
#include <sys/time.h>

#include "services.h"

/************** Global variables ****************/
int     debug   	= 0;
int     nofork 		= 1;
int     pid		= 0;
int     quit    	= 0;    	/* Set to 1 if we are to quit */
int     chanserv        = 0;
int     nickserv        = 0;
int     memoserv        = 0;
char    *quitmsg        = NULL;         /* Contains a message as to why service is terminating */
char    inbuf[1024];
char    serr[2048];
char    *ServerName;
int 	sock;


int main(int ac, char **av, char **envp)
{
  char **s_buf;
  int buf_c;
  struct timeval timeout;
  fd_set readfds, writefds;
  int ret;  
  int cmd_send = 1;  
  int i;
  int link_ok = 0;
  
  /*Setting up some signal handlerz */
  signal(SIGHUP, sighup_handler);     /* SIGHUP handler */
  /*
   signal(SIGTERM, sigterm_handler);
   signal(SIGUSR2, sigusr2_handler);
   */
  
  /* Parse the commandline options */

  close(STDIN_FILENO);
  
  if (parse_options(ac, av))
    exit(1);
  if (strlen(getconfigvalue("CONFIGFILE")) == 0)
    {
      push_configvalue("CONFIGFILE", "/home/lanvall/projects/ircd-pi-services/stuff/services.conf");
    }
  
  message("Reading config file (%s)\t", getconfigvalue("CONFIGFILE"));
  if (!readconfig())
    {
      printfail();
      error("%s", serr);
      nofork = 1;
      exit(0);
    } else
    printok();
  
  
  message("Open Logfile (%s)\t\t", getconfigvalue("LOGFILE"));
  if (!open_log())
    {
      printfail();
      error("%s", serr);
      
    } else
      printok();
  
  /* Try to make a connection to MySQL Server */
  message("Connecting to MySQL Server....\t\t\t");
  if (!database_connect())
    {
      printfail();
      error("%s", serr);
      exit(0);
    } else
      printok();
  
  message("Reading config from MySQL Server\t\t");
  if (!readmysqlconf())
    {
      printfail();
      error("%s", serr);
      exit(0);
    } else
      printok();
  
  /* Dumping out MySQL config */
  dumpmysqlconf();
  
  
  if (getmysqlconfigvalue("SSL") != NULL && strcmp(getmysqlconfigvalue("SSL"), "1") == 0)
    {
      message("Initializing SSL...\n");
      if (!ssl_init())
        {
          printfail();
          fatal("\t\tCan't inizialize SSL shit");
        } 
      else
        {
          printok();
        }
    }
  
  
  /* Connecting to IRC Server */
  message("Connection to IRC Server...\t\t\t");
  if (getmysqlconfigvalue("ircserver") == NULL)
    {
      printfail();
      fatal("Missing 'ircserver' parameter");
    }
  
  if (getmysqlconfigvalue("ircserverport") == NULL)
    {
      printfail();
      fatal("Missing 'ircserverport' parameter");
    }
  if (getmysqlconfigvalue("ircpassword") == NULL)
    {
      printfail();
      fatal("Missing 'ircpassword' parameter");
    }
  if (getmysqlconfigvalue("servername") == NULL)
    {
      printfail();
      fatal("Missing 'servername' parameter");
    }  else
    ServerName = getmysqlconfigvalue("servername");
  if (getmysqlconfigvalue("description") == NULL)
    {
      printfail();
      fatal("Missing 'description' parameter");
    }
  
  sock = conn(getmysqlconfigvalue("ircserver"), atoi(getmysqlconfigvalue("ircserverport")));
  if (sock == 0)
    {
      printfail();
      exit(1);
    } else {
      printok();
      message("Linking with IRC Server...\t\t\t");
      while(1)
        {
          
          if (!sock)
            {
              printf("BLAHHHHHHHHHHHHHHH");
              exit(5);
            }
          
          timeout.tv_sec = 1; 
          timeout.tv_usec = 0;
          
          FD_ZERO(&readfds);
          FD_ZERO(&writefds); 
          
          FD_SET(sock, &readfds);
          
          if(cmd_send)
            FD_SET(sock, &writefds);
          
          if(select(sock + 1, &readfds, &writefds, NULL, &timeout) <= 0)
            continue;
          
          if (FD_ISSET(sock, &readfds))
            {
              inbuf[0] = '\0';
              ret = recv(sock, inbuf, 1024, 0);
              if (ret > 0)
                {
                  inbuf[ret] = '\0';
                  //fprintf(stderr, "inbuf: %s\n", inbuf);
                  
                  if (strnicmp(inbuf, "ERROR", 5) == 0)
                    {
                      printfail();
                      fprintf(stderr, "%s\n", inbuf);
                      exit(1);
                    }
                  
                
                  buf_c = split_buf(inbuf, &s_buf, 0);
                  if (buf_c)
                    {
                      for (i = 0; i < buf_c; i++)
                        if (strnicmp(s_buf[i], "SERVER", 6) == 0)
                          {
                            link_ok = 1;
                            break;
                          }
                      
                    }
                  
                  if (link_ok)
                    {
                      printok();
                      break;
                    }
                  
                                   
                  if (strnicmp(inbuf, "NOTICE", 6) == 0)
                    fprintf(stderr, "%s\n", inbuf);
                }
              
              if (ret == -1)
                {
                  printok();
                  break;
                  //continue;
                }
              
            }
          
          if (FD_ISSET(sock, &writefds))
            {
              if (cmd_send)
                {
                  send_cmd(NULL, "PASS %s :TS", getmysqlconfigvalue("ircpassword"));
                  send_cmd(NULL, "SERVER %s 1 :%s", getmysqlconfigvalue("servername"), getmysqlconfigvalue("description"));
                  send_cmd(NULL, "SVINFO 3 3 0 :%ld", time(NULL));
                  send_cmd(NULL, "CAPAB TS3 SSJOIN");
                  cmd_send = 0;
                  //break;
                }
              
              continue;
            }          
        }
    }
  
  message("Starting ChanServ Service... \t\t\t");
  if (getmysqlconfigvalue("EnableChanServ") != NULL && strcmp(getmysqlconfigvalue("EnableChanServ"), "1") == 0)
    {
      send_cmd(NULL, "NICK ChanServ 1 %d +oe ChanServ %s %s ChanServ-Service", time(NULL), ServerName, ServerName);
      chanserv = 1;
      printok();
    }
  else
    printwarn();
  
  message("Starting NickServ Service... \t\t\t");
  if (getmysqlconfigvalue("EnableNickServ") != NULL && strcmp(getmysqlconfigvalue("EnableNickServ"), "1") == 0)
    {
      send_cmd(NULL, "NICK NickServ 1 %d +oe NickServ %s %s NickServ-Service", time(NULL), ServerName, ServerName);
      nickserv = 1;
      printok();
    }
  else
    printwarn();
  
  message("Starting MemoServ Service... \t\t\t");
  if (getmysqlconfigvalue("EnableMemoServ") != NULL && strcmp(getmysqlconfigvalue("EnableMemoServ"), "1") == 0)
    {
      send_cmd(NULL, "NICK MemoServ 1 %d +oe MemoServ %s %s MemoServ-Service", time(NULL), ServerName, ServerName);
      memoserv = 1;
      printok();
    }
  else
    printwarn();
  
  /* Forking...*/
  if (!nofork)
    {
      pid = fork();
      debugmsg(1, "PID is %d", pid);
      if (pid)
        {
          writepid(pid);
          exit(0);
        }
    }
  

  printf("\n\nSOCK %d\n\n", sock);
  
  while(1)
    {
      timeout.tv_sec = 1; 
      timeout.tv_usec = 0;
      
      FD_ZERO(&readfds);
      FD_ZERO(&writefds); 
      
      FD_SET(sock, &readfds);
      
      if(cmd_send)
        FD_SET(sock, &writefds);
      
      if(select(sock + 1, &readfds, &writefds, NULL, &timeout) <= 0)
        continue;
      
      if (FD_ISSET(sock, &readfds))
        {
          inbuf[0] = '\0';
          ret = recv(sock, inbuf, 1024, 0);
          if (ret > 0)
            {
              inbuf[ret] = '\0';
              fprintf(stderr, "%s\n", inbuf);
            }
        }
      
      if (FD_ISSET(sock, &writefds))
        {
      
        }          
    }
  
  
  
  
  /* Main Loop */
  /*
   n = fopen("/dev/null", "w");
   while(1)
    {
      fputs("BLAH", n);
      if (quit)
        break;
      nickserv_check_kill_timer();
      //sgets2(inbuf, sizeof(inbuf), sock);
      process();
      inbuf[0] = '\0';
    }
  fclose(n);
  */
  close (sock);
  return 0;
}

