/*
* Copyright (C) 2002 Alain Stucki <astucki@myisp.ch>
*                    Roman Senn   <roman@dreamlab.ch>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Id: nickserv_timer.c,v 1.4 2002/07/31 21:17:55 astucki Exp $
*/

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <netdb.h>
#include <netinet/in.h>
#include <setjmp.h>
#include <sys/socket.h>
#include <sys/stat.h>	/* for umask() on some systems */
#include <sys/types.h>
#include <sys/time.h>
#include <malloc.h>

#include "services.h"
#include "message.h"

struct NickServ_Timer *rootnickservtimer = NULL;

/*====================*/
/* Get Struct by Name */
/*====================*/
struct NickServ_Timer *gettimerptr(char *name)
{
  struct NickServ_Timer *timerptr;
  for(timerptr = rootnickservtimer; timerptr; timerptr = timerptr->next)
    {
      if (strcmp(timerptr->nick, name) == 0)
        return timerptr;
    }
  return NULL;
}

/*=======================================*/
/* Returns a pointer to next User struct */
/*=======================================*/
struct NickServ_Timer *fetch_nickservtimer()
{
  struct NickServ_Timer *timer = rootnickservtimer;
  
  if (timer == NULL)
    {
      timer = calloc(1, sizeof(struct NickServ_Timer));
      timer->prev = NULL;
    }
  else
    {
      while(timer->next != NULL)
        timer = timer->next;
      
      timer->next = calloc(1, sizeof(struct NickServ_Timer));
      timer->next->prev = timer;
      timer = timer->next;
    }
  timer->next = NULL;
  
  return timer;
  
}

/*======================*/
/* Destroy Timer Struct */
/*======================*/
void destroy_timer(struct NickServ_Timer *timer)
{
  if (timer != NULL)
    {
      if (timer == rootnickservtimer)
        {
          rootnickservtimer = rootnickservtimer->next;
          if (rootnickservtimer != NULL)
            rootnickservtimer->prev = NULL;
        } else {
          timer->prev->next = timer->next;
          if (timer->next != NULL)
            timer->next->prev = timer->prev;
        }
      free(timer);
    }
}

/*============================================================*/
/* Checks for runed off timers (40,20, 10 and 0 seconds) */
/*============================================================*/
void nickserv_check_kill_timer()
{
  struct NickServ_Timer *timerptr;
  
  
  for(timerptr = rootnickservtimer; timerptr; timerptr = timerptr->next)
        {
          if (timerptr->added + 20 < time(NULL) && timerptr->warn1 == 0)
            {
              /* Send 40 Seconds warning */
              send_cmd(NULL, ":NickServ NOTICE %s :You now have 40 seconds to comply before you get killed!", timerptr->nick);
              timerptr->warn1 = 1;
            }
          if (timerptr->added + 40 <= time(NULL) && timerptr->warn2 == 0)
            {
              /* Send 40 Seconds warning */
              send_cmd(NULL, ":NickServ NOTICE %s :You now have 20 seconds to comply before you get killed!", timerptr->nick);
              timerptr->warn2 = 1;
            }
          if (timerptr->added + 50 <= time(NULL) && timerptr->warn3 == 0)
            {
              /* Send 40 Seconds warning */
              send_cmd(NULL, ":NickServ NOTICE %s :You now have 10 seconds to comply before you get killed!", timerptr->nick);
              timerptr->warn3 = 1;
            }
                if (timerptr->added + 60 <= time(NULL) && timerptr->warn4 == 0)
            {
              timerptr->warn4 = 1;
              send_cmd(NULL, "KILL %s", timerptr->nick);
              timerptr->die = 1;
              /* TODO: add free for timer */
            }
        }
  return;
}


/*=========================*/
/* Adds a timer for a user */
/*=========================*/
void addkilltimer(char *nick)
{
  struct NickServ_Timer *timerptr_check = NULL;
  struct NickServ_Timer *timerptr = calloc(1, sizeof(struct NickServ_Timer));
  
  timerptr_check = gettimerptr(nick);
  
  if (timerptr_check != NULL)   /*Timer exists */
    return;
  
  timerptr = fetch_nickservtimer();
  
  timerptr->added = time(NULL);
  timerptr->warn1 = 0;
  timerptr->warn2 = 0;
  timerptr->warn3 = 0;
  timerptr->warn4 = 0;
  timerptr->die = 0;
  strncpy(timerptr->nick, nick, 255);
  if (rootnickservtimer == NULL)
    {
      rootnickservtimer = timerptr;
      timerptr->prev = NULL;
    }
  return;
}
