/*
* Copyright (C) 2002 Alain Stucki <astucki@myisp.ch>
*                    Roman Senn   <roman@dreamlab.ch>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Id: sockio.c,v 1.11 2002/08/03 20:04:17 astucki Exp $
*/


#include <stdio.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <openssl/ssl.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/select.h>
#include <string.h>
#include <mysql.h>

int sock;
int debug;

int conn(char *hostname, unsigned short port);
int set_non_blocking(int fd);


int conn(char *hostname, unsigned short port)
{
  int fd;
  int ret;
  struct hostent *host;
  struct protoent *proto;
  struct sockaddr_in addr;
  struct timeval timeout; 
  fd_set readfds, writefds, exceptfds;
  
  if (hostname == NULL)
    {
      fprintf(stderr, "No hostname given\n");
      exit(1);
    }
  
  if (!port)
    {
      fprintf(stderr, "No port given\n");
      exit(1);
    }
   
  host = gethostbyname(hostname);
  proto = getprotobyname("tcp");
  
  addr.sin_family = AF_INET;
  addr.sin_port = htons(port);
  addr.sin_addr.s_addr = *(unsigned int *)host->h_addr;

  fd = socket(AF_INET, SOCK_STREAM, proto->p_proto);
  if (fd < 0)
    {
      fprintf(stderr, "Can't create socket\n");
      exit(1);
    }
  
  if (!set_non_blocking(fd))
    {
      fprintf(stderr, "Can't set non-blocking\n");
      exit(1);
    }
   
  ret = connect(fd, (struct sockaddr *)&addr, sizeof(struct sockaddr_in));
  
  while(1)
    {
      
      timeout.tv_sec = 10; 
      timeout.tv_usec = 0;
      
      FD_ZERO(&readfds); 
      FD_ZERO(&writefds); 
      FD_ZERO(&exceptfds); 
      
      FD_SET(fd, &readfds); 
      FD_SET(fd, &writefds); 
      FD_SET(fd, &exceptfds);
      
      select(fd + 1, &readfds, &writefds, &exceptfds, &timeout);
      
      if (FD_ISSET(fd, &writefds))
        return fd;
      
      if (FD_ISSET(fd, &exceptfds))
        return 0;
    }
}



int set_non_blocking(int fd)
{
  int res;
  int nonb = 0;
  
  nonb |= O_NONBLOCK;
  res = fcntl(fd, F_GETFL, 0);
  if (-1 == res || fcntl(fd, F_SETFL, res | nonb) == -1)
    return 0;
  return 1;
}


void send_cmd(const char *source, const char *fmt, ...)
{
  char buffer[4096];
  va_list args;
  va_start(args, fmt);
  
  vsnprintf(buffer, sizeof(buffer) - 2, fmt, args);
  strcat(buffer, "\r\n");
  
  fprintf(stderr, "Sock: %d String: %s\n", sock, buffer);
  send(sock, buffer, strlen(buffer), 0);
  va_end(args);
}


