/*
* Copyright (C) 2002 Alain Stucki <astucki@myisp.ch>
*                    Roman Senn   <roman@dreamlab.ch>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of the
* License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* $Id: users.c,v 1.9 2002/07/31 21:17:55 astucki Exp $
*/

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mysql.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include <netdb.h>
#include <netinet/in.h>
#include <setjmp.h>
#include <sys/socket.h>
#include <sys/stat.h>	/* for umask() on some systems */
#include <sys/types.h>
#include <sys/time.h>
#include <malloc.h>


#include "services.h"
#include "message.h"

struct User *rootuser = NULL;

/*====================*/
/* Get Struct by Name */
/*====================*/
struct User *getuserptr(char *name)
{
  struct User *userptr;
  for(userptr = rootuser; userptr; userptr = userptr->next)
    {
      if (strcmp(userptr->nickname, name) == 0)
        return userptr;
    }
  return NULL;
}

/*=======================================*/
/* Returns a pointer to next User struct */
/*=======================================*/
struct User *fetch_user()
{
  struct User *client = rootuser;
  
  if (client == NULL)
    {
      client = calloc(1, sizeof(struct User));
      client->prev = NULL;
    }
  else
    {
      while(client->next != NULL)
        client = client->next;
      
      client->next = calloc(1, sizeof(struct User));
      client->next->prev = client;
      client = client->next;
    }
  client->next = NULL;
  
  return client;
  
}

/*=======================*/
/* Destroy Client Struct */
/*=======================*/
void destroy_user(struct User *user)
{
  if (user != NULL)
    {
      if (user == rootuser)
        {
          rootuser = rootuser->next;
          if (rootuser != NULL)
            rootuser->prev = NULL;
        } else {
          user->prev->next = user->next;
          if (user->next != NULL)
            user->next->prev = user->prev;
        }
      free(user);
    }
}


/*==========================*/
/* Handler for NICK command */
/*==========================*/
void m_nick(char *source, int ac, char **av)
{
  struct User *userptr = calloc(1, sizeof(struct User));
  struct User *chuserptr = NULL;
  int i;
  char nick[strlen(source) + 1];
  
  if (ac == 8)
    {
      userptr = fetch_user();
      strncpy(userptr->nickname, av[0], 255);
      strncpy(userptr->username, av[4], 255);
      strncpy(userptr->host, av[5], 255);
      strncpy(userptr->modes, av[3], 255);
      strncpy(userptr->server, av[6], 255);
      strncpy(userptr->fullname, av[7], 4196);
      userptr->hopcount = atoi(av[1]);
      userptr->ident = 0;
      if (rootuser == NULL)
        {
          rootuser = userptr;
          userptr->prev = NULL;
        }
      debugmsg(1, "added user %s to User struct", av[0]);
      nickserv_newnick(av[0]);
      return;
      
    }
  else if (ac == 2)
    {
      for (i = 0; i < strlen(source); i++)
        if (source[i] == ' ')
          nick[i] = '\0';
      else
        nick[i] = source[i];
      
      chuserptr = getuserptr(nick);
      if (chuserptr == NULL)
        return;
      strncpy(chuserptr->nickname, av[0], 255);
      chuserptr->ident = 0;
      nickserv_changenick(nick, av[0], atoi(av[1]));
    }
  else
    {
      error("Wrong Parameter Count for NICK (%d)", ac);
      free (userptr);
      return;
    }
}

/*==========================*/
/* Handler for QUIT command */
/*==========================*/
void m_quit(char *source, int ac, char **av)
{
  char nick[strlen(source) + 1];
  struct User *userptr = NULL;
  int i;
  
  
  for (i = 0; i < strlen(source); i++)
    if (source[i] == ' ')
      nick[i] = '\0';
  else
    nick[i] = source[i];
  
  userptr = getuserptr(nick);
  destroy_user(userptr);
  return;
  
}
